"""Main module file of YamlBase"""

import os
import yaml
from YamlBase.dbworkers import SQLiteWorker
from YamlBase.table_representation import Table
from YamlBase.yaml_worker import YamlDataBaseWorker, YamlActionsWorker
from YamlBase.exceptions import InsertActionImpossibleError, DataBaseTypeNotFound, ActionNotFound, \
    RemoveActionImpossibleError


class YBase:
    """Main class of YamlBase"""

    cache_folder = "YamlBaseCache"

    def __init__(self, path_to_yaml_db):
        """
        :param path_to_yaml_db: path to yaml representation of DataBase
        """
        self.yaml_config = YamlDataBaseWorker(path_to_yaml_db)
        self.create_cache_folder_if_not_exist()
        folder_path = '/'.join(path_to_yaml_db.split('/')[:-1]) + '/'
        self.actions_config = YamlActionsWorker(folder_path+self.yaml_config.db_info['db_name'])
        self.db_worker = self.initialize_worker()

    def create_cache_folder_if_not_exist(self):
        if not os.path.exists(self.cache_folder):
            os.mkdir(self.cache_folder)

    def initialize_worker(self):
        if self.yaml_config.db_info['db_type'].lower() == 'sqlite':
            return SQLiteWorker(self.yaml_config)
        else:
            raise DataBaseTypeNotFound(self.yaml_config.db_info['db_type'])

    def do_actions(self):
        for table, action in self.actions_config.action_table.items():
            try:
                self.do_action(action, self._filter_table_by_tablename(table))
                print(f"\nSUCCESS ACTION {action} on table {table}\n")
            except Exception as e:
                print(f"\nFAILED ACTION {action} on table {table}\nReason: {e}\n")

    def _filter_table_by_tablename(self, table_name):
        return [i for i in self.yaml_config.tables_info if i.table_name == table_name][0]

    def do_action(self, action, table_name):
        """
        :param action:
        :param table_name:
        :return:
        """
        if action == 'insert':
            return self.insert_action(table_name)
        elif action == 'remove':
            return self.remove_action(table_name)

        raise ActionNotFound(action)

    def check_action_possibility(self, action: str, table_obj: Table):
        action_is_possible = True
        if action == 'insert':
            if not self.db_worker.check_insert_table_possibility(table_obj.table_name):
                action_is_possible = False
                # Condition for check optimization
                if not action_is_possible:
                    return action_is_possible
        if action == 'remove':
            if not self.db_worker.check_remove_table_possibility(table_obj.table_name):
                action_is_possible = False
                # Condition for check optimization
                if not action_is_possible:
                    return action_is_possible
        return action_is_possible

    def insert_action(self, new_table: Table):
        if not self.check_action_possibility('insert', new_table):
            raise InsertActionImpossibleError()

        self.db_worker.insert_new_table(new_table)

    def remove_action(self, remove_table: Table):
        if not self.check_action_possibility('remove', remove_table):
            raise RemoveActionImpossibleError()

        self.db_worker.remove_table(remove_table)

    def init_config_action(self):
        return self.db_worker.save_config()



"""Module of custom exceptions"""


class FileReadError(Exception):
    """Custom exception"""
    def __init__(self, filename):
        super().__init__(f"Error was occurred while reading {filename}")


class FieldNotFoundError(Exception):
    """Custom exception"""
    def __init__(self, field_name):
        super().__init__(f"Field: '{field_name}' not found")


class InsertActionImpossibleError(Exception):
    """Custom exception"""
    def __init__(self):
        super().__init__("Can't do insert action")


class RemoveActionImpossibleError(Exception):
    """Custom exception"""
    def __init__(self):
        super().__init__("Can't do remove action")


class DataBaseTypeNotFound(Exception):
    """Custom exception"""
    def __init__(self, db_name):
        super().__init__(f"DataBase type '{db_name}' not supported yet")


class ActionNotFound(Exception):
    """Custom exception"""
    def __init__(self, action):
        super().__init__(f"Action '{action}' not supported yet")


class ActionConfigSyntaxError(Exception):
    """Custom exception"""
    def __init__(self):
        super().__init__(f"Syntax error in action config file")



import sys
from sqlalchemy import create_engine
import os
from YamlBase.sql import PostgreSQL
from YamlBase.yaml_worker import YamlDataBaseWorker
import pytest


class TestPostgreSQL:

    def setup(self):
        test_db_ip = 'ec2-54-228-250-82.eu-west-1.compute.amazonaws.com:5432'
        test_db = 'd5h3q2hrorhk5d'
        test_user = 'mzhrcgiqucfnxv'
        test_password = 'bba3716ee4b3af0bd724333fb22afed02d588371c149b5685f9f62e7e2029b24'

        self.postgresql_connector = PostgreSQL(test_db_ip, test_db, test_user, test_password)

        self.cfg = YamlDataBaseWorker("./test/test_base_postgre.yml")

    def test_empty_base(self):
        """Check if base is empty"""
        # If some test failed then it should be table in database
        if self.postgresql_connector.get_table_list():
            for table in self.postgresql_connector.get_table_list():
                self.postgresql_connector.remove_table(table.schema, table.table)
        assert not self.postgresql_connector.get_table_list()

    def test_add_remove_schema(self):
        assert set(self.postgresql_connector.get_schemas_list()) == {'public', 'pg_catalog', 'information_schema'}
        self.postgresql_connector.create_schema('test_schema')
        assert set(self.postgresql_connector.get_schemas_list()) == {'public', 'pg_catalog', 'information_schema',
                                                                     'test_schema'}
        self.postgresql_connector.remove_schema('test_schema')
        assert set(self.postgresql_connector.get_schemas_list()) == {'public', 'pg_catalog', 'information_schema'}

    def test_add_remove_table(self):
        """Add and remove table to sqlite DB"""
        self.postgresql_connector.create_table(self.cfg.tables_info[0])
        assert len(self.postgresql_connector.get_table_list())
        self.postgresql_connector.remove_table('public', 'table1')
        assert not self.postgresql_connector.get_table_list()

    def test_get_table_data(self):

        self.postgresql_connector.create_table(self.cfg.tables_info[0])

        table_data = self.postgresql_connector.get_table_data(self.cfg.tables_info[0].schema_name,
                                                              self.cfg.tables_info[0].table_name)
        assert table_data['name'] == self.cfg.tables_info[0].table_name
        assert table_data['schema'] == self.cfg.tables_info[0].schema_name

        self.postgresql_connector.remove_table(self.cfg.tables_info[0].schema_name, self.cfg.tables_info[0].table_name)

    def test_get_table_schema_dict(self):
        self.postgresql_connector.create_table(self.cfg.tables_info[0])

        assert self.postgresql_connector.get_table_schema_dict() == {self.cfg.tables_info[0].schema_name:
                                                                         self.cfg.tables_info[0].table_name}

        self.postgresql_connector.remove_table(self.cfg.tables_info[0].schema_name, self.cfg.tables_info[0].table_name)



from .table_representation import Table, Column


def convert_table_data_from_dict_to_table_obj(table_dict: dict, schema_name: str) -> Table:

    columns = []

    for col in table_dict['columns']:

        is_pk = table_dict['columns'][col]['is_pk'] == 1

        is_sk = table_dict['columns'][col]['is_sk'] == 1
        sk_link = None

        if is_sk:
            sk_link = table_dict['columns'][col]['link']

        columns.append(
            Column(
                table_dict['columns'][col]['name'],
                table_dict['columns'][col]['type'],
                is_pk,
                is_sk,
                sk_link
            )
        )

    return Table(
        schema_name=schema_name,
        table_name=table_dict['name'],
        permissions=table_dict['permissions'],
        columns=columns
    )




import os
import yaml
import pytest
import sys

@pytest.fixture
def test_insert_config():
    return {
        'table1' : 'insert',
        'table2' : 'insert'
    }

@pytest.fixture
def test_remove_config():
    return {
        'table1' : 'remove',
        'table2' : 'remove'
    }


def test_insert_pipeline(test_insert_config, test_remove_config):
    os.system("python setup.py sdist")
    filename = os.listdir('dist')[0]
    os.system("pip install dist/" + filename)
    with open("test/configs_for_tests/test_db_actions.yaml", 'w') as f: 
        yaml.dump(test_insert_config, f)

    os.system('yamlbase -cfg_db test/configs_for_tests/test_base_pipeline.yaml')
    
    with open("YamlBaseCache/test__SQLite_test_db.yaml") as f:
        cfg = yaml.load(f, Loader=yaml.Loader)

    assert list(cfg['schemas'].keys()) == ['main']
    assert cfg['schemas']['main'][0]['name'] == 'table1'
    assert cfg['schemas']['main'][1]['name'] == 'table2'

    with open("test/configs_for_tests/test_db_actions.yaml", 'w') as f: 
        yaml.dump(test_remove_config, f)

    os.system('yamlbase -cfg_db test/configs_for_tests/test_base_pipeline.yaml')
    
    with open("YamlBaseCache/test__SQLite_test_db.yaml") as f:
        cfg = yaml.load(f, Loader=yaml.Loader)
    
    assert not cfg['schemas']['main']

    os.system("pip uninstall sdist/" + filename)


"""Main file of YamlBase"""

import argparse
import yaml
import os
from YamlBase.ybase import YBase
import sys
import os


def main():

    parser = argparse.ArgumentParser(description='This utility allows you to manage changes to the database using '
                                                 'configurations')
    parser.add_argument('-cfg_db', action='store', dest='cfg_path',
                        help='Path to yaml file with DB config')

    args = parser.parse_args()
    base = YBase(args.cfg_path)
    base.do_actions()


if __name__ == '__main__':
    main()


from sqlalchemy import create_engine
import pandas as pd
from YamlBase.table_representation import Table
from .sql import SQL
from .sql import table_schema
from typing import List


class PostgreSQL(SQL):

    excluded_schemas = ['pg_catalog', 'information_schema']

    def __init__(self, host, db_name, user, password):
        super().__init__()
        self.conn = create_engine(f"postgresql://{user}:{password}@{host}/{db_name}")

    def get_schemas_list(self) -> list:
        tt = pd.read_sql("select schema_name from information_schema.schemata;", self.conn)
        return list(tt['schema_name'].unique())

    def get_table_list(self) -> List[table_schema]:
        temp = pd.read_sql("SELECT * FROM information_schema.tables", self.conn)
        temp = temp[~temp['table_schema'].isin(self.excluded_schemas)]
        result = []
        for table_name, schema_name in zip(temp['table_name'], temp['table_schema']):
            result.append(table_schema(table_name, schema_name))
        return result

    def create_schema(self, schema_name):
        self.conn.execute(f"create schema {schema_name}")

    def remove_schema(self, schema_name, cascade=False):
        if not cascade:
            self.conn.execute(f"drop schema {schema_name}")
        else:
            self.conn.execute(f"drop schema {schema_name} CASCADE")

    def create_table(self, table_data: Table, if_exists_check=False):
        query = f"CREATE TABLE {table_data.schema_name}.{table_data.table_name}"
        if if_exists_check:
            query += "IF NOT EXISTS"
        query += "("

        for col in table_data.columns:
            query += f"{col.column_name} {col.column_type}"
            if col.is_pk:
                query += " PRIMARY KEY"
            query += ","

        # Removing last comma
        query = query[:-1]

        query += ")"
        self.conn.execute(query)

    def remove_table(self, schema, table_name):
        query = f"DROP TABLE {schema}.{table_name}"
        self.conn.execute(query)

    def _get_pk_list(self, table_name, schema_name) -> list:
        query = f"""SELECT
              pg_attribute.attname,
              format_type(pg_attribute.atttypid, pg_attribute.atttypmod)
            FROM pg_index, pg_class, pg_attribute, pg_namespace
            WHERE
              pg_class.oid = '{table_name}'::regclass AND
              indrelid = pg_class.oid AND
              nspname = '{schema_name}' AND
              pg_class.relnamespace = pg_namespace.oid AND
              pg_attribute.attrelid = pg_class.oid AND
              pg_attribute.attnum = any(pg_index.indkey)"""
        # query = ''
        return list(pd.read_sql(query, self.conn)['attname'].values)

    def get_table_data(self, schema, table_name) -> dict:
        query = f"select * from information_schema.columns where table_schema = '{schema}' and table_name  = '{table_name}'"
        temp = pd.read_sql(query, self.conn)[['column_name', 'data_type', 'is_nullable']]
        pk_list = self._get_pk_list(table_name, schema)

        table_data = dict()
        table_data['name'] = table_name
        table_data['schema'] = schema
        table_data['permissions'] = []
        table_data['columns'] = {}

        for col in temp['column_name'].values:
            table_data['columns'][col] = {
                'name': col,
                # list() and type transform are necessary because values by default
                # returns np object and spoils yaml
                'type': str(list(temp[temp['column_name'] == col]['data_type'].values)[0]),
                'is_nullable': str(list(temp[temp['column_name'] == col]['is_nullable'].values)[0]),
                'is_pk': col in pk_list
            }
        return table_data

    def get_table_schema_dict(self):
        temp = pd.read_sql(f"select * from information_schema.tables", self.conn)[['table_schema', 'table_name']]
        temp = temp[temp['table_schema'] != 'pg_catalog']
        temp = temp[temp['table_schema'] != 'information_schema']
        return {k: v for k, v in zip(temp['table_schema'], temp['table_name'])}


from YamlBase import YBase
import pytest
from YamlBase.dbworkers import SQLiteWorker
from YamlBase import ActionNotFound
from YamlBase import DataBaseTypeNotFound
from YamlBase import InsertActionImpossibleError, RemoveActionImpossibleError


class TestYBase:

    def test_initialize_sqlite_worker(self):
        obj = YBase('./test/test_base.yml')
        assert isinstance(obj.db_worker, SQLiteWorker)

        with pytest.raises(FileNotFoundError):
            YBase('randomfile.yaml')

        with pytest.raises(DataBaseTypeNotFound):
            YBase('test/configs_for_tests/test_ybase/test_miss_db_type.yml')

    def test_filter_table_by_tablename(self):
        obj = YBase('./test/test_base.yml')
        assert obj._filter_table_by_tablename('table1') == obj.yaml_config.tables_info[0]

    def test_insert_remove_action(self):
        obj = YBase('./test/test_base.yml')
        # Check for empty database
        assert not obj.db_worker.tables

        obj.insert_action(obj.yaml_config.tables_info[0])

        with pytest.raises(InsertActionImpossibleError):
            obj.insert_action(obj.yaml_config.tables_info[0])

        assert obj.db_worker.tables[0] == obj.yaml_config.tables_info[0].table_name

        obj.remove_action(obj.yaml_config.tables_info[0])

        with pytest.raises(RemoveActionImpossibleError):
            obj.remove_action(obj.yaml_config.tables_info[0])

        assert not obj.db_worker.tables

    def test_check_action_possibility(self):
        obj = YBase('./test/test_base.yml')

        assert not obj.check_action_possibility('remove', obj.yaml_config.tables_info[0])
        assert obj.check_action_possibility('insert', obj.yaml_config.tables_info[0])

        obj.insert_action(obj.yaml_config.tables_info[0])

        assert obj.check_action_possibility('remove', obj.yaml_config.tables_info[0])
        assert not obj.check_action_possibility('insert', obj.yaml_config.tables_info[0])

        obj.remove_action(obj.yaml_config.tables_info[0])

    def test_do_actions(self, capsys):
        obj = YBase('./test/test_base.yml')
        # Replace actions config with custom for test case
        obj.actions_config.action_table = {'table1': 'insert'}

        with pytest.raises(ActionNotFound):
            obj.do_action('add', 'table1')

        assert not obj.db_worker.tables

        obj.do_actions()

        assert obj.db_worker.tables[0] == obj.yaml_config.tables_info[0].table_name

        obj.actions_config.action_table = {'table1': 'remove'}
        obj.do_actions()
        capsys.readouterr()
        obj.do_actions()

        out, err = capsys.readouterr()
        assert out == "\nFAILED ACTION remove on table table1\nReason: Can't do remove action\n\n"
        assert not obj.db_worker.tables



from YamlBase import Table, Column
import pytest


@pytest.fixture
def test_column_1():
    return Column(
        column_name='column1',
        column_type='text',
        is_pk=True,
        is_sk=False
    )


@pytest.fixture
def test_column_2():
    return Column(
        column_name='column2',
        column_type='int',
        is_pk=False,
        is_sk=False
    )


@pytest.fixture
def test_permission():
    return {
        'user1': 'r*w*',
        'user2': 'r'
    }


class TestTable:

    def test_eq_right_order(self, test_column_1, test_column_2, test_permission):
        table1 = Table(
            schema_name='schema1',
            table_name='table1',
            columns=[test_column_1, test_column_2],
            permissions=test_permission
        )

        table2 = Table(
            schema_name='schema1',
            table_name='table1',
            columns=[test_column_1, test_column_2],
            permissions=test_permission
        )

        assert table1 == table2

    def test_eq_wrong_col_order(self, test_column_1, test_column_2, test_permission):
        table1 = Table(
            schema_name='schema1',
            table_name='table1',
            columns=[test_column_1, test_column_2],
            permissions=test_permission
        )

        table2 = Table(
            schema_name='schema1',
            table_name='table1',
            columns=[test_column_2, test_column_1],
            permissions=test_permission
        )

        assert table1 == table2


class TestColumns:

    def test_str(self, test_column_1):
        assert f"column_name = column1\ncolumn_type = text\n" \
               f"is_pk = True\nis_sk = False\nsk_link = None" == str(test_column_1)




import sys
from sqlalchemy import create_engine
import os

from YamlBase.sql import SQLite
from YamlBase.yaml_worker import YamlDataBaseWorker
import pytest


class TestSQLite:

    def setup(self):
        self.sqlite_connector = SQLite('', './test/test_db.db')
        self.cfg = YamlDataBaseWorker("./test/test_base.yml")

    def test_empty_base(self):
        """Check if base is empty"""
        assert not self.sqlite_connector.get_table_list()

    def test_add_remove_table(self):
        """Add and remove table to sqlite DB"""
        self.sqlite_connector.create_table(self.cfg.tables_info[0])
        assert len(self.sqlite_connector.get_table_list())
        self.sqlite_connector.remove_table('table1')
        assert not self.sqlite_connector.get_table_list()

    def test_get_table_data(self):

        self.sqlite_connector.create_table(self.cfg.tables_info[0])

        print(self.cfg.tables_info[0])

        assert self.sqlite_connector.get_table_data(self.cfg.tables_info[0].schema_name,
                                             self.cfg.tables_info[0].table_name)['name'] == self.cfg.tables_info[0].table_name

        self.sqlite_connector.remove_table(self.cfg.tables_info[0].table_name)

    def test_get_table_schema_dict(self):
        self.sqlite_connector.create_table(self.cfg.tables_info[0])

        assert self.sqlite_connector.get_table_schema_dict() == {'main': self.cfg.tables_info[0].table_name}

        self.sqlite_connector.remove_table(self.cfg.tables_info[0].table_name)



import sys
from sqlalchemy import create_engine

from YamlBase.dbworkers import SQLiteWorker
from YamlBase.yaml_worker import YamlDataBaseWorker
import pytest


class TestSQLiteWorker:

    def setup(self):
        self.cfg = YamlDataBaseWorker("./test/test_base.yml")
        self.sqlite_connector = SQLiteWorker(self.cfg)

    def test_remove_tables(self):
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])

        assert not self.sqlite_connector.tables

    def test_insert_new_tables(self):
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])
        assert set(self.sqlite_connector.tables) <= set([i.table_name for i in self.cfg.tables_info])

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])

    def test_check_insert_table_possibility(self):
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])

        # Not to insert table that already exists
        assert not self.sqlite_connector.check_insert_table_possibility(self.cfg.tables_info[0].table_name)

        assert self.sqlite_connector.check_insert_table_possibility("test_name")

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])

    def test_check_remove_table_possibility(self):
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])

        # Not to insert table that already exists
        assert self.sqlite_connector.check_remove_table_possibility(self.cfg.tables_info[0].table_name)

        assert not self.sqlite_connector.check_remove_table_possibility("test_name")

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])

    def test_read_base_information_from_database(self):
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])
        self.sqlite_connector.read_base_information_from_database()

        assert self.sqlite_connector.schemas == ['main']
        assert self.cfg.tables_info[0].table_name in self.sqlite_connector.tables

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])

    def test_save_config(self):

        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])

        self.sqlite_connector.read_base_information_from_config('YamlBaseCache/_SQLite_test_db.yaml')

        assert len(self.sqlite_connector.tables) == 1
        assert len(self.sqlite_connector.schemas) == 1
        assert self.sqlite_connector.tables[0] == 'table1'
        assert self.sqlite_connector.schemas[0] == 'main'

        self.sqlite_connector.insert_new_table(self.cfg.tables_info[1])

        assert len(self.sqlite_connector.tables) == 2
        assert self.sqlite_connector.tables[1] == 'table2'

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])
        self.sqlite_connector.remove_table(self.cfg.tables_info[1])

        assert len(self.sqlite_connector.tables) == 0

    def test_read_base_information_from_config(self):
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[0])
        self.sqlite_connector.insert_new_table(self.cfg.tables_info[1])

        # Clear old data
        self.sqlite_connector.tables = []
        self.sqlite_connector.schemas = []

        self.sqlite_connector.read_base_information_from_config('YamlBaseCache/_SQLite_test_db.yaml')

        assert len(self.sqlite_connector.tables) == 2
        assert len(self.sqlite_connector.schemas) == 1

        assert self.sqlite_connector.tables[0] == 'table1'
        assert self.sqlite_connector.tables[1] == 'table2'
        assert self.sqlite_connector.schemas[0] == 'main'

        self.sqlite_connector.remove_table(self.cfg.tables_info[0])
        self.sqlite_connector.remove_table(self.cfg.tables_info[1])



from YamlBase.yaml_worker import YamlDataBaseWorker, YamlActionsWorker, ActionConfigSyntaxError
from YamlBase.exceptions import FieldNotFoundError
import pytest
import os
from YamlBase.table_representation import Table, Column


@pytest.fixture
def test_no_fields_folder():
    return "./test/configs_for_tests/test_no_fields/"


@pytest.fixture
def test_action_folder():
    return "./test/configs_for_tests/test_actions/"


@pytest.fixture
def correct_first_column_object():
    return Column(
        "column1",
        "text",
        1,
        0
    )


@pytest.fixture
def correct_second_column_object():
    return Column(
        "column2",
        "double",
        0,
        1,
        'table2'
    )


@pytest.fixture
def correct_third_column_object():
    return Column(
        "column3",
        "text",
        0,
        0
    )


@pytest.fixture
def correct_first_table_permissions():
    return {
        "usr1": "r*",
        "usr2": "w*"
    }


class TestYamlDatabaseWorker:

    def test_read_yaml_base_config_file_not_found(self):
        with pytest.raises(FileNotFoundError):
            YamlDataBaseWorker('some_file.yaml')

    def test_read_yaml_base_config_field_not_found(self, test_no_fields_folder):

        for f in os.listdir(test_no_fields_folder):

            with pytest.raises(FieldNotFoundError):
                YamlDataBaseWorker(test_no_fields_folder + f)

    def test_parse_yaml(self, correct_first_column_object, correct_second_column_object,
                        correct_third_column_object, correct_first_table_permissions):

        obj = YamlDataBaseWorker('./test/test_base.yml')

        assert obj.tables_info[0].columns[0] == correct_first_column_object
        assert obj.tables_info[0].columns[1] == correct_second_column_object
        assert obj.tables_info[0].columns[2] == correct_third_column_object

        assert obj.tables_info[0].table_name == 'table1'
        assert obj.tables_info[0].permissions == correct_first_table_permissions


class TestYamlActionsWorker:

    def test_read_yaml_base_config_file_not_found(self):
        with pytest.raises(FileNotFoundError):
            YamlActionsWorker('some_file.yaml')

    def test_data_parsing(self):
        obj = YamlActionsWorker('./test/test_db')
        assert obj.action_table == {
            "table1": "insert",
            "table2": "remove"
        }

    def test_check_syntax(self, test_action_folder):
        with pytest.raises(ActionConfigSyntaxError):
            YamlActionsWorker(test_action_folder + 'test_db')



"""Main module fo YamlBase"""

from .ybase import YBase
from .exceptions import ActionNotFound, DataBaseTypeNotFound, ActionConfigSyntaxError, FieldNotFoundError, \
    RemoveActionImpossibleError, FileReadError, InsertActionImpossibleError
from .utils import convert_table_data_from_dict_to_table_obj
from .table_representation import Table
from .table_representation import Column
from .yaml_worker import YamlActionsWorker
from .yaml_worker import YamlDataBaseWorker
from .sql import SQLite
from .sql import SQL
from .yamlbase import main

__all__ = [
    'YamlDataBaseWorker',
    'YamlActionsWorker',
    'Table',
    'Column',
    'YBase',
    'ActionNotFound',
    'DataBaseTypeNotFound',
    'ActionConfigSyntaxError',
    'FieldNotFoundError',
    'RemoveActionImpossibleError',
    'FileReadError',
    'InsertActionImpossibleError',
    'convert_table_data_from_dict_to_table_obj',
    'SQL',
    'SQLite',
    'main'
]


from __future__ import absolute_import

import os
import sys
import sqlite3

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src/')))

import YamlBase

if os.path.exists('./test/test_db'):
    os.remove('./test/test_db')

sqlite3.connect('./test/test_db')


import sys
from sqlalchemy import create_engine

from YamlBase.dbworkers import PostgreWorker
from YamlBase.yaml_worker import YamlDataBaseWorker
import pytest


class TestPostgreWorker:

    def setup(self):
        self.cfg = YamlDataBaseWorker("./test/test_base_postgre.yml")
        self.postgre_connector = PostgreWorker(self.cfg)

    def test_remove_tables(self):
        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])

        self.postgre_connector.remove_table(self.cfg.tables_info[0])

        assert not self.postgre_connector.tables

    def test_insert_new_tables(self):
        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])
        assert set(self.postgre_connector.tables) <= set([i.table_name for i in self.cfg.tables_info])

        self.postgre_connector.remove_table(self.cfg.tables_info[0])

    def test_check_insert_table_possibility(self):
        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])

        # Not to insert table that already exists
        assert not self.postgre_connector.check_insert_table_possibility(self.cfg.tables_info[0].table_name)

        assert self.postgre_connector.check_insert_table_possibility("test_name")

        self.postgre_connector.remove_table(self.cfg.tables_info[0])

    def test_check_remove_table_possibility(self):
        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])

        # Not to insert table that already exists
        assert self.postgre_connector.check_remove_table_possibility(self.cfg.tables_info[0].table_name)

        assert not self.postgre_connector.check_remove_table_possibility("test_name")

        self.postgre_connector.remove_table(self.cfg.tables_info[0])

    def test_read_base_information_from_database(self):
        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])
        self.postgre_connector.read_base_information_from_database()

        assert set(self.postgre_connector.schemas) == {'pg_catalog', 'information_schema', 'public'}
        assert self.cfg.tables_info[0].table_name in self.postgre_connector.tables

        self.postgre_connector.remove_table(self.cfg.tables_info[0])

    def test_save_config(self):

        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])

        self.postgre_connector.read_base_information_from_config('YamlBaseCache/ec2-54-228-250-82.eu-west-1.compute.'
                                                                 'amazonaws.com_5432_SQLite_d5h3q2hrorhk5d.yaml')

        assert len(self.postgre_connector.tables) == 1
        assert len(self.postgre_connector.schemas) == 3
        # assert self.postgre_connector.tables[0] == 'table1'
        assert 'public' in self.postgre_connector.schemas

        self.postgre_connector.insert_new_table(self.cfg.tables_info[1])

        assert len(self.postgre_connector.tables) == 2
        assert 'table2' in self.postgre_connector.tables

        self.postgre_connector.remove_table(self.cfg.tables_info[0])
        self.postgre_connector.remove_table(self.cfg.tables_info[1])

        assert len(self.postgre_connector.tables) == 0

    def test_read_base_information_from_config(self):
        self.postgre_connector.insert_new_table(self.cfg.tables_info[0])
        self.postgre_connector.insert_new_table(self.cfg.tables_info[1])

        # Clear old data
        self.postgre_connector.tables = []
        self.postgre_connector.schemas = []

        self.postgre_connector.read_base_information_from_config('YamlBaseCache/ec2-54-228-250-82.eu-west-1.compute.'
                                                                 'amazonaws.com_5432_SQLite_d5h3q2hrorhk5d.yaml')

        assert len(self.postgre_connector.tables) == 2
        assert len(self.postgre_connector.schemas) == 3

        assert 'table1' in self.postgre_connector.tables and 'table2' in self.postgre_connector.tables

        assert 'public' in self.postgre_connector.schemas

        self.postgre_connector.remove_table(self.cfg.tables_info[0])
        self.postgre_connector.remove_table(self.cfg.tables_info[1])



"""Custom classes to present table/column from config or table"""


from dataclasses import dataclass


@dataclass
class Column:
    """class to present column from config or table"""
    column_name: str
    column_type: str
    is_pk = bool
    is_sk = bool
    sk_link = str

    def __init__(self, column_name, column_type, is_pk, is_sk, sk_link=None):
        self.column_name = column_name
        self.column_type = column_type
        self.is_pk = is_pk
        self.is_sk = is_sk
        self.sk_link = sk_link

    def __str__(self):

        return f"column_name = {self.column_name}\ncolumn_type = {self.column_type}\n" \
               f"is_pk = {self.is_pk}\nis_sk = {self.is_sk}\nsk_link = {self.sk_link}"


@dataclass
class Table:
    """class to table column from config or table"""
    schema_name: str
    table_name: str
    columns: list
    permissions: dict

    def __eq__(self, other):
        return self.table_name == other.table_name and self.schema_name == other.schema_name and \
               all([i in other.columns for i in self.columns])



from YamlBase.sql import SQLite, SQL


class DBWorker:
    cache_folder = "YamlBaseCache"

    def __init__(self, yaml):
        pass

    def check_if_used(self):
        pass

    def initialize_connection(self) -> SQL:
        pass

    def read_base_information_from_database(self):
        pass

    def read_base_information_from_config(self, cfg_path):
        pass

    def check_base_updates(self):
        pass

    def remove_table(self, table_name):
        pass

    def insert_new_table(self, new_table):
        pass

    def save_config(self):
        pass

    def check_insert_table_possibility(self, new_table_name):
        pass

    def create_db_identifier(self) -> str:
        return f"{self.db_data.db_info['ip']}_{self.db_data.db_info['db_type']}_" \
                             f"{self.db_data.db_info['db_name']}".replace('/', '_').replace(':', '_')



from .base import DBWorker
from YamlBase.yaml_worker import YamlDataBaseWorker
import os
import yaml
from YamlBase.sql import SQLite


class SQLiteWorker(DBWorker):
    tables: list
    schemas: list

    def __init__(self, yaml_db: YamlDataBaseWorker):
        super().__init__(yaml_db)
        self.db_data = yaml_db
        self.conn = self.initialize_connection()
        self.db_identifier = self.create_db_identifier()
        if not self.check_if_used():
            self.read_base_information_from_database()
            self.save_config()
        else:
            self.read_base_information_from_config(self.cache_folder + '/' + self.db_identifier + '.yaml')

    def check_if_used(self):
        return self.db_identifier + '.yaml' in os.listdir(self.cache_folder)

    def initialize_connection(self) -> SQLite:
        return SQLite(self.db_data.db_info['ip'], self.db_data.db_info['db_name'])

    def insert_new_table(self, new_table):
        self.conn.create_table(new_table)
        self.tables = [i.table for i in self.conn.get_table_list()]
        self.save_config()

    def remove_table(self, table_obj):
        self.conn.remove_table(table_obj.table_name)
        self.tables = [i.table for i in self.conn.get_table_list()]
        self.save_config()

    def read_base_information_from_database(self):
        self.schemas = self.conn.get_schemas_list()
        self.tables = [i.table for i in self.conn.get_table_list()]

    def read_base_information_from_config(self, cfg_path):
        with open(cfg_path, 'r') as f:
            cfg = yaml.load(f, yaml.Loader)
        self.schemas = list(cfg['schemas'].keys())
        self.tables = []
        for schema in self.schemas:
            for table in cfg['schemas'][schema]:
                self.tables.append(table['name'])

    def save_config(self) -> dict:
        cfg = dict()
        cfg.update(self.db_data.db_info)

        cfg['schemas'] = {}
        cfg['schemas']['main'] = []
        for table in self.tables:
            table_data = self.conn.get_table_data('main', table)
            cfg['schemas']['main'].append(table_data)

        with open(self.cache_folder + '/' + self.db_identifier + '.yaml', 'w') as f:
            yaml.dump(cfg, f)

    def check_insert_table_possibility(self, new_table_name) -> bool:
        """
        :param new_table_name:name of new table
        :return: true if inserting is possible
        """
        # Check if table already exists
        for table in self.tables:
            if table == new_table_name:
                return False

        # Check if all tables that exists in base - defined in config
        for table in self.conn.get_table_schema_dict().values():
            if table not in [i.table_name for i in self.db_data.tables_info]:
                return False

        return True

    def check_remove_table_possibility(self, table_name_to_remove) -> bool:
        """
        :param table_name_to_remove: name of table to check
        :return: if table can be removed
        """
        return table_name_to_remove in [i.table for i in self.conn.get_table_list()]
